/**
 * This file contains some code adapted from https://github.com/alexeski/tableau-embed-viz-responsive-apiv3.
 * See: https://alexeskinasy.medium.com/responsive-design-best-practices-for-embedding-tableau-304f12509a5d
 */

(function (Drupal, once) {
  /**
   *  Adjust the visualization size and scale to fit the available space.
   *
   * @param {string} deviceType
   * @param {array} props
   */
  function scaleViz(props) {
    if (props.debug) {
      console.log("scaling...");
    }
    // This is the element that will be scaled down.
    const container = props.viz.closest(".tableauViz-container");
    props.currentWidth = props.vizDiv.offsetWidth;

    if (props.mobileBreakpoint >= props.currentWidth) {
      if (props.debug) {
        console.log(
          "Mobile rendering: ",
          props.mobileBreakpoint,
          " >= ",
          props.currentWidth
        );
      }

      props.deviceType = "phone";
      // if device change (from desktop/default to phone), load 'phone' version and scale
      if (props.viz.device === "desktop" || props.viz.device === "default") {
        props.deviceType = "phone";
        props.viz.device = props.deviceType;

        // if no device changes, then just scale
      } else {
        props.deviceType = "phone";
      }
    }
    // size change from phone to desktop
    else if (props.viz.device === "phone") {
      props.deviceType = "desktop";
      props.viz.device = props.deviceType;

      // if no device chang, then just scale
    } else {
      props.deviceType = "desktop";
    }

    if (props.behaviour === "range") {
      if (props.debug) {
        console.log(
          "Your dashboard is configured to use a Range, so you are on your own, we're not going to scale it. That's ok for mobile layouts, but for large screes we recommend using automatic or fixed size dashboards!"
        );
      }
    } else if (props.behaviour === "exactly") {
      // sets the origin to scale from
      container.style.transformOrigin = "left top";

      // simple scaling factor
      // e.g. if currentWidth = 900 and maxWidth = 1200, scaling factor is .75
      let scalingFactor = props.currentWidth / props.maxWidth;

      // caps scaling factor so it only scales down; does not scale up (increase size of viz)
      if (scalingFactor > 1) {
        scalingFactor = 1;
      }

      // scaling is essentially disabled for mobile; will always be 1
      if (props.deviceType === "phone") {
        scalingFactor = 1;
      }

      // Finally scale the viz using the scalingFactor derived above
      // log current parameters
      if (props.debug) {
        console.log(
          `Current width: ${props.currentWidth}; maxWidth: ${props.maxWidth}; scalling Factor: ${scalingFactor}; device= ${props.viz.device}`
        );
      }

      // apply CSS transform
      container.style.transform = `scale(${scalingFactor})`;

      // Reserve the necessary vertical space.
      container.style.minHeight = `${
        props.viz.offsetHeight * scalingFactor
          ? props.viz.offsetHeight * scalingFactor
          : props.viz.offsetHeight
      }px`;

      // Tweak Flexbox to ensure the Tableau iframe content is always
      // centered correctly to the page, regardless of scaling or device
      // type.
      props.viz.style.justifyContent = "center";
      if (scalingFactor === 1) {
        props.viz.style.position = "relative";
      } else {
        props.viz.style.position = "absolute";
      }

      if (props.debug) {
        console.log("position", props.viz.style.position);
      }
    }
  }

  /**
   * Checks if the visualization is a dashboard or a standalone worksheet, and
   * scales it depending on some factors.
   *
   * @param {object} event
   *   The event fired.
   */
  function tableauReady(viz) {
    const debug = false;
    // Get the reference size from a container that doesn't grow with if the
    // content is wider (overflow: hidden).
    const vizDiv = viz.closest(".tableauViz-wrapper");
    const currentWidth = vizDiv.offsetWidth;
    const mobileBreakpoint = 650;
    let maxWidth; // this is the max width as defined in the dashboard with Fixed size.
    const { activeSheet } = viz.workbook;
    const behaviour =
      activeSheet._workbookImpl._activeSheetImpl._sheetInfoImpl._sheetSize
        .behavior;

    if (debug) {
      console.log("Viz has loaded!");
      console.log("Active sheet:", activeSheet.name);
      console.log("Worksheet type:", activeSheet.sheetType);
      console.log("Workbook name:", viz.workbook.name);
    }

    const deviceType = "";

    // Determine what type of sheet it is, worksheet, dashboard or story.
    switch (activeSheet.sheetType) {
      case "dashboard":
        if (debug) {
          console.log("Found Dashboard!");
        }

        // check dashboard size settings (exactly vs range vs automatic )
        // note: this function isn't implemented in the api, so we need to access it via the prototype
        // note2: width will not exist and an error will be thrown if size isn't set to exactly or range.
        if (behaviour === "exactly") {
          maxWidth =
            activeSheet._workbookImpl._activeSheetImpl._sheetInfoImpl._sheetSize
              .maxSize.width;
        }

        scaleViz({
          deviceType,
          viz,
          behaviour,
          debug,
          vizDiv,
          maxWidth,
          currentWidth,
          mobileBreakpoint,
        });
        break;
      default:
        // Other options can be "story" or "worksheet";
        if (debug) {
          console.log("Unsupported sheet type: ", activeSheet.sheetType);
        }
        break;
    }

    // Adjust the viz when the container changes width.
    new ResizeObserver(() => {
      scaleViz({
        viz,
        vizDiv,
        debug,
        behaviour,
        maxWidth,
        mobileBreakpoint,
      });
    }).observe(vizDiv);
  }

  Drupal.behaviors.media_tableau = {
    attach(context) {
      const tableauViz = once("media_tableau", "tableau-viz", context);
      tableauViz.forEach((viz) => {
        viz.toolbar = "hidden";
        viz.addEventListener("firstinteractive", (event) => {
          tableauReady(event.target);
        });
      });
    },
  };
})(Drupal, once);
